@@ -146,6 +146,7 @@ class CustomCanvas(tk.Canvas):
         self.bind("<Double-Button-1>", self.cb_lclick2)
         self.bind("<Button-3>", self.cb_rclick)
         self.bind("<B3-Motion>", self.cb_rmotion)
+        self.bind("<Double-Button-3>", self.cb_rclick2)
         self.bind("<Alt-1>", self.cb_altlclick)
         self.bind("<Alt-3>", self.cb_altrclick)
         self.bind("<Shift-3>", self.cb_shiftrclick)
@@ -224,6 +225,11 @@ class CustomCanvas(tk.Canvas):
     def cb_rclick(self, event):
         self.cache["pos"] = self.master.get_selected_imgs()[0].affine.translate()
 
+    def cb_rclick2(self, event):
+        myimg = self.master.get_selected_imgs()[0]
+        myimg.reset_affine()
+        self.parent.parent.refresh()
+
     @backend.cache_event()
     def cb_rmotion(self, event):
         raise Exception()
