@@ -829,7 +829,7 @@ class NDArray(ImgTemplate):
     Subclass of NumPy array which permits use of package while maintaing as close to standard array behavior as possible
     """
 
-    _base_header = {}
+    _base_header = {"descr": "<i2", "fortran_order": False, "shape": (0,)}
 
     defaults = {
         "shape": 0,
@@ -860,9 +860,23 @@ class NDArray(ImgTemplate):
 
     def _read_protocol(self):
         if self.filename is None:
-            pass
+            return
+        with open(f"{self.fileloc}{self.filename}", "rb") as f:
+            fbytes = f.read(10)
+            mstring = fbytes[:6]
+            v_maj = fbytes[6]
+            v_min = fbytes[7]
+            header_len = fbytes[8] + fbytes[9] * 256
+            header_bytes = f.read(header_len)
+            self.header = eval(header_bytes)
         return self
 
+    def _fileshape(self):
+        if self.filename is None:
+            return self.shape
+        else:
+            return tuple(self.header["shape"])
+
     def _load_protocol(self):
         npy = np.load(self.fileloc + self.filename)
         self.dtype = npy.dtype
@@ -1018,6 +1032,12 @@ class AIM(ImgTemplate):
         super().__array_finalize__(*args, **kwargs)
         self._header_update()
 
+    def _fileshape(self):
+        if self.filename is None:
+            return self.shape
+        else:
+            return tuple(np.flip(self.header["dim"]))
+
     def _read_protocol(self):
         if self.filename is None:
             return self
@@ -1099,8 +1119,8 @@ class AIM(ImgTemplate):
                 self[i].ravel()[:] = np.fromfile(
                     f, dtype=self.dtype, count=np.prod(self.shape[1:])
                 )
-            if self.verbosity >= 2:  # pragma: no cover
-                print("\n")
+            # if self.verbosity >= 2:  # pragma: no cover
+            #     print("\n")
         self.position.ravel()[:] = (np.array(self.shape) - 1) // 2 * self.affine.scale()
 
         return self
@@ -1290,7 +1310,7 @@ class AIM(ImgTemplate):
                 f"Identified version {self._version()} has no interpretation."
             )
 
-        return AffineTensor(self.ndim).scale(*np.flip(elsize))
+        return AffineTensor(len(elsize)).scale(*np.flip(elsize))
 
     def _affine2elsize(self):
         """
